startInSpring = {}
startInSpring.debugActive = false

function startInSpring:load(filename)
	Environment.INITIAL_DAY = 1 --//Sets INITIAL_DAY to change the start month. Default is 6(August). (Desired Month - 2)
end

Environment.load = Utils.prependedFunction(Environment.load, startInSpring.load)

function FieldManager:loadMapData(xmlFile)
	FieldManager:superClass().loadMapData(self)

	local mission = g_currentMission
	self.mission = mission

	mission:addUpdateable(self)

	local terrainNode = mission.terrainRootNode
	local fieldGroundSystem = mission.fieldGroundSystem
	local sprayLevelMapId, sprayLevelFirstChannel, sprayLevelNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.SPRAY_LEVEL)
	local sprayLevelMaxValue = fieldGroundSystem:getMaxValue(FieldDensityMap.SPRAY_LEVEL)
	local plowLevelMapId, plowLevelFirstChannel, plowLevelNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.PLOW_LEVEL)
	local plowLevelMaxValue = fieldGroundSystem:getMaxValue(FieldDensityMap.PLOW_LEVEL)
	self.limeLevelMaxValue = 0
	self.plowLevelMaxValue = plowLevelMaxValue
	self.sprayLevelMaxValue = sprayLevelMaxValue
	self.fruitModifiers = {}
	self.sprayLevelModifier = DensityMapModifier.new(sprayLevelMapId, sprayLevelFirstChannel, sprayLevelNumChannels, terrainNode)
	self.plowLevelModifier = DensityMapModifier.new(plowLevelMapId, plowLevelFirstChannel, plowLevelNumChannels, terrainNode)

	if Platform.gameplay.useLimeCounter then
		local limeLevelMapId, limeLevelFirstChannel, limeLevelNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.LIME_LEVEL)
		self.limeLevelMaxValue = fieldGroundSystem:getMaxValue(FieldDensityMap.LIME_LEVEL)
		self.limeLevelModifier = DensityMapModifier.new(limeLevelMapId, limeLevelFirstChannel, limeLevelNumChannels, terrainNode)
	end

	if Platform.gameplay.useStubbleShred then
		local stubbleShredLevelMapId, stubbleShredLevelFirstChannel, stubbleShredLevelNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.STUBBLE_SHRED)
		self.stubbleShredModifier = DensityMapModifier.new(stubbleShredLevelMapId, stubbleShredLevelFirstChannel, stubbleShredLevelNumChannels, terrainNode)
	end

	local groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.GROUND_TYPE)
	self.groundTypeModifier = DensityMapModifier.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels, terrainNode)
	local groundAngleMapId, groundAngleFirstChannel, groundAngleNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.GROUND_ANGLE)
	self.angleModifier = DensityMapModifier.new(groundAngleMapId, groundAngleFirstChannel, groundAngleNumChannels, terrainNode)
	local sprayTypeMapId, sprayTypeFirstChannel, sprayTypeNumChannels = fieldGroundSystem:getDensityMapData(FieldDensityMap.SPRAY_TYPE)
	self.sprayTypeModifier = DensityMapModifier.new(sprayTypeMapId, sprayTypeFirstChannel, sprayTypeNumChannels, terrainNode)
	self.fieldFilter = DensityMapFilter.new(groundTypeMapId, groundTypeFirstChannel, groundTypeNumChannels)

	self.fieldFilter:setValueCompareParams(DensityValueCompareType.GREATER, 0)

	if mission.weedSystem:getMapHasWeed() then
		local weedMapId, weedFirstChannel, weedNumChannels = mission.weedSystem:getDensityMapData()
		self.weedModifier = DensityMapModifier.new(weedMapId, weedFirstChannel, weedNumChannels, terrainNode)
	end

	self.fieldGroundSystem = fieldGroundSystem
	local terrainDetailHeightId = mission.terrainDetailHeightId
	self.terrainHeightTypeModifier = DensityMapModifier.new(terrainDetailHeightId, g_densityMapHeightManager.heightTypeFirstChannel, g_densityMapHeightManager.heightTypeNumChannels)
	self.terrainHeightModifier = DensityMapModifier.new(terrainDetailHeightId, getDensityMapHeightFirstChannel(terrainDetailHeightId), getDensityMapHeightNumChannels(terrainDetailHeightId))
	self.groundTypeSown = fieldGroundSystem:getFieldGroundValue(FieldGroundType.SOWN)
	self.sprayTypeFertilizer = fieldGroundSystem:getFieldSprayValue(FieldSprayType.FERTILIZER)
	self.sprayTypeLime = fieldGroundSystem:getFieldSprayValue(FieldSprayType.LIME)
	self.availableFruitTypeIndices = {}
	self.availableGrainFruitTypeIndices = {}

	--//startInSpring: Build table of crops that standard grainHeader's can harvest.
	for _, grainFruitIndex in pairs(g_fruitTypeManager:getFruitTypesByCategoryNames('grainHeader')) do
		local isPlantingSeason = g_currentMission.growthSystem:canFruitBePlanted(grainFruitIndex)
		local grainFruitType = g_fruitTypeManager:getFruitTypeByIndex(grainFruitIndex)
		if grainFruitType.useForFieldJob and grainFruitType.allowsSeeding and grainFruitType.needsSeeding and isPlantingSeason then
			table.insert(self.availableGrainFruitTypeIndices, grainFruitIndex)
		end
	end

	--//startInSpring: Build table of crops that can be planted during selected season.
	for _, fruitType in ipairs(g_fruitTypeManager:getFruitTypes()) do
		local isPlantingSeason = g_currentMission.growthSystem:canFruitBePlanted(fruitType.index)
		if fruitType.useForFieldJob and fruitType.allowsSeeding and fruitType.needsSeeding and isPlantingSeason then
			table.insert(self.availableFruitTypeIndices, fruitType.index)
		end
	end

	self.fruitTypesCount = #self.availableFruitTypeIndices
	self.fieldIndexToCheck = 1

	g_asyncTaskManager:addSubtask(function()
		for i, field in ipairs(self.fields) do
			local posX, posZ = field:getCenterOfFieldWorldPosition() --/ added local
			local farmland = g_farmlandManager:getFarmlandAtWorldPosition(posX, posZ)
			if farmland ~= nil then
				field:setFarmland(farmland)
				if self.farmlandIdFieldMapping[farmland.id] == nil then
					self.farmlandIdFieldMapping[farmland.id] = {}
				end
				table.insert(self.farmlandIdFieldMapping[farmland.id], field)
			else
				Logging.error("Failed to find farmland in center of field '%s'", i)
			end
		end
	end)

	if not mission.missionInfo.isValid and g_server ~= nil then
		g_asyncTaskManager:addSubtask(function()
			--//startInSpring: Function - findFieldFruit
			local function findFieldFruit(field)
				--DebugUtil.printTableRecursively(field, "", 0, 0)
				field.fruitType = 0
				local x, z = field:getCenterOfFieldWorldPosition()
				local function testFruit(varFruitType)
					local minState = 0
					local maxState = 15
					local area, totalArea = FieldUtil.getFruitArea(x - 1, z - 1, x + 1, z - 1, x - 1, z + 1, FieldUtil.FILTER_EMPTY, FieldUtil.FILTER_EMPTY, varFruitType, minState, maxState, 0, 0, 0, false)
					print(area)
					print(totalArea)
					return area > 0.5 * totalArea
				end


				for _, fruitType in ipairs(g_fruitTypeManager:getFruitTypes()) do
					if testFruit(fruitType.index) then
						field.fruitType = fruitType.index
					end
				end
				if testFruit(FruitType.GRASS) then
					field.fruitType = FruitType.GRASS
				end
				startInSpring:debug('findFieldFruit/field:' .. field.fieldId .. '/fruitType:' .. field.fruitType)
			end

			--//startInSpring: Function - applyFruitToField
			local function applyFruitToField(field, fruitIndex)
				startInSpring:debug('applyFruitToField/field:' .. field.fieldId .. '/fruitType:' .. fruitIndex)
				if field.fieldGrassMission then
					fruitIndex = FruitType.GRASS
				end
				field.plannedFruit = fruitIndex
				local fieldState = FieldManager.FIELDSTATE_GROWING
				local plowState = nil
				if not mission.missionInfo.plowingRequiredEnabled then
					plowState = self.plowLevelMaxValue
				else
					plowState = math.random(0, self.plowLevelMaxValue)
				end
				local sprayLevel = math.random(0, self.sprayLevelMaxValue)
				local limeState = math.random(0, self.limeLevelMaxValue)
				local weedValue = 0
				if fruitIndex == 0 then
					fieldStates = { FieldManager.FIELDSTATE_PLOWED, FieldManager.FIELDSTATE_CULTIVATED,
						FieldManager.FIELDSTATE_HARVESTED }
					fieldState = fieldStates[math.random(#fieldStates)]
					if fieldState == FieldManager.FIELDSTATE_HARVESTED then
						fruitIndex = self.availableFruitTypeIndices[math.random(1, #self.availableFruitTypeIndices)]
						fruitDesc = g_fruitTypeManager:getFruitTypeByIndex(fruitIndex)
						growthState = fruitDesc.cutState
					elseif fieldState == FieldManager.FIELDSTATE_PLOWED then
						growthState = FieldManager.FIELDSTATE_PLOWED
						plowState = self.plowLevelMaxValue
					elseif fieldState == FieldManager.FIELDSTATE_CULTIVATED then
						growthState = FieldManager.FIELDSTATE_CULTIVATED
					end
				elseif fruitIndex == FruitType.GRASS then
					growthState = FieldManager.FIELDEVENT_GROWN
				else
					growthState = 1
					local fruitData = g_currentMission.growthSystem.seasonalFruitData[fruitIndex]
					if fruitData ~= nil then
						if fruitData[12].plantingAllowed then
							growthState = 2
						elseif fruitData[11].plantingAllowed then
							growthState = 3
						end
					end
					startInSpring:debug('setFruitGrowth/field:' ..
					field.fieldId .. '/fruitType:' .. fruitIndex .. '/growthState:' .. growthState)
				end
				for i = 1, table.getn(field.maxFieldStatusPartitions) do
					self:setFieldPartitionStatus(field, field.maxFieldStatusPartitions, i, fruitIndex, fieldState,
						growthState, sprayLevel, false, plowState, weedValue, limeState)
				end
			end

			--//startInSpring: endFunctions
			for _, field in pairs(self.fields) do
				-- findFieldFruit(field)
				field.fruitType = self.availableFruitTypeIndices[math.random(#self.availableFruitTypeIndices)]

				startInSpring:debug('findFieldFruitReturned/field:' .. field.fieldId .. '/fruitType:' .. field.fruitType)
				if field:getIsAIActive() and field.farmland ~= nil then
					--//startInSpring: Handle GE planted fruits.
					if field.farmland.isOwned and field.fruitType ~= nil and field.fruitType ~= FruitType.GRASS and field.fruitType ~= FruitType.UNKNOWN then
						local isPlantingSeason = g_currentMission.growthSystem:canFruitBePlanted(field.fruitType)
						if not isPlantingSeason then
							fruitIndex = self.availableGrainFruitTypeIndices
							[math.random(1, #self.availableGrainFruitTypeIndices)]
							startInSpring:debug('plantingCheckFailed/field:' ..
							field.fieldId ..
							'/fruitType:' ..
							g_fruitTypeManager:getFruitTypeNameByIndex(field.fruitType) .. ' Planting: ' .. fruitIndex)
						else
							fruitIndex = field.fruitType
							startInSpring:debug('plantingCheckPassed/field:' ..
							field.fieldId .. '/fruitType:' .. g_fruitTypeManager:getFruitTypeNameByIndex(field.fruitType))
						end
					elseif not field.farmland.isOwned and field.fruitType ~= nil and field.fruitType ~= FruitType.GRASS and field.fruitType ~= FruitType.UNKNOWN then
						local isPlantingSeason = g_currentMission.growthSystem:canFruitBePlanted(field.fruitType)
						if not isPlantingSeason then
							fruitIndex = self.availableFruitTypeIndices[math.random(1, #self.availableFruitTypeIndices)]
							startInSpring:debug('plantingCheckFailed/field:' ..
							field.fieldId ..
							'/fruitType:' ..
							g_fruitTypeManager:getFruitTypeNameByIndex(field.fruitType) .. ' Planting: ' .. fruitIndex)
						else
							fruitIndex = field.fruitType
							startInSpring:debug('plantingCheckPassed/field:' ..
							field.fieldId .. '/fruitType:' .. g_fruitTypeManager:getFruitTypeNameByIndex(fruitIndex))
						end
					elseif not field.farmland.isOwned and field.fruitType == FruitType.GRASS then
						fruitIndex = FruitType.GRASS
						startInSpring:debug('plantingCheckGrass/field:' ..
						field.fieldId .. '/fruitType:' .. g_fruitTypeManager:getFruitTypeNameByIndex(fruitIndex))
					else
						fruitIndex = 0
						startInSpring:debug('plantingCheckEmpty/field:' .. field.fieldId .. '/fruitType:0')
					end
					--//startInSpring: diceRoll for empty fields.
					if fruitIndex == 0 and field.fieldMissionAllowed then
						grassFieldBias = math.random(1, 7)
						if grassFieldBias == 1 then
							fruitIndex = FruitType.GRASS
						elseif grassFieldBias == math.random(2, 4) then
							if field.farmland.isOwned then
								fruitIndex = self.availableGrainFruitTypeIndices
								[math.random(1, #self.availableGrainFruitTypeIndices)]
							else
								fruitIndex = self.availableFruitTypeIndices
								[math.random(1, #self.availableFruitTypeIndices)]
							end
						end
						startInSpring:debug('diceRollEmptyField/field:' .. field.fieldId .. '/fruitType:' .. fruitIndex)
					elseif fruitIndex == 0 and not field.fieldMissionAllowed then
						startInSpring:debug('diceRollNotAllowed/field:' .. field.fieldId .. '/fruitType:0')
					end
				end
				--//startInSpring: Handle diceRoll results.
				applyFruitToField(field, fruitIndex)
			end
		end)
	elseif g_server ~= nil then
		for _, field in pairs(self.fields) do
			g_asyncTaskManager:addSubtask(function()
				self:findFieldFruit(field)
			end)
		end
	end
	g_asyncTaskManager:addSubtask(function()
		self:findFieldSizes()
	end)
	g_asyncTaskManager:addSubtask(function()
		g_farmlandManager:addStateChangeListener(self)
		if mission:getIsServer() and g_addCheatCommands then
			addConsoleCommand("gsFieldSetFruit", "Sets a given fruit to field", "consoleCommandSetFieldFruit", self)
			addConsoleCommand("gsFieldSetFruitAll", "Sets a given fruit to all fields", "consoleCommandSetFieldFruitAll",
				self)
			addConsoleCommand("gsFieldSetGround", "Sets a given fruit to field", "consoleCommandSetFieldGround", self)
			addConsoleCommand("gsFieldSetGroundAll", "Sets a given fruit to allfield", "consoleCommandSetFieldGroundAll",
				self)
		end
		if g_addCheatCommands then
			addConsoleCommand("gsFieldToggleStatus", "Shows field status", "consoleCommandToggleDebugFieldStatus", self)
		end
	end)
	g_asyncTaskManager:addSubtask(function()
		if not mission:getIsServer() then
			for _, field in pairs(self.fields) do
				self:setFieldGround(field, FieldGroundType.CULTIVATED, field.fieldAngle, 0, 0, 0, 0, 0, 0, false, false)
			end
		end
	end)
	g_messageCenter:subscribe(MessageType.FARM_PROPERTY_CHANGED, self.onFarmPropertyChanged, self)
	g_messageCenter:subscribe(MessageType.YEAR_CHANGED, self.onYearChanged, self)
	g_messageCenter:subscribe(MessageType.PERIOD_CHANGED, self.onPeriodChanged, self)
end

function Mission00:getIsTourSupported()
	--return self.missionInfo.difficulty == 1 or GS_IS_MOBILE_VERSION --//Disable guidedTour since crops are not harvestable.
	return false
end

function startInSpring:load()
	self.firstRun = 0
end

function startInSpring:draw()
	if g_currentMission.missionInfo.saveDate == nil and self.firstRun ~= 1 then
		g_gui:showInfoDialog({
			text = g_i18n:getText("startInSpring_FIRSTRUN")
		})
	end
	self.firstRun = 1
end

function startInSpring:debug(msg)
	if startInSpring.debugActive then
		print(tostring('DEBUG: startInSpring/' .. msg))
	end
end

addModEventListener(startInSpring)
